#!/usr/bin/env python

'''
@author: szhelev

If this code works, it was written by Svetoslav Zhelev. If not then I don't know who wrote it.
'''

import pygtk
from time import mktime, time, strptime
from CERNNotifier import CERNUtils
from CERNNotifier import CERNConfiguration
from CERNNotifier import CERNNotifierFeedException
from CERNNotifier import CERNNotifierFeedAuthException
import threading
pygtk.require('2.0')
import gtk
import gobject
import CERNNotifier
import webbrowser
import pickle
import sys
import os.path

try:
    from gtkwin32 import *
except:
    pass

try:
    import pynotify
    if not pynotify.init('CERNNotifier'):
        pynotify = False
except:
    pynotify = False

if not pynotify:
    CERNConfiguration.getInstance().getLogger().warn('Notice: pynotify could not be loaded. Balloon notifications will be unavailable.')


is_windows = sys.platform.startswith('win')		
WM_LBUTTONUP = 0x0202
WM_RBUTTONUP = 0x0205

# some constants definition
ABOUT_ICON = 'icons/cern.jpg'
DEFAULT_ICON = 'icons/notifier.png'
NEW_MSG_ICON = 'icons/new_msg.png'
BUSY_ICON = 'icons/busy.png'
DEFAULT_ICON_WIN = 'icons/notifier.ico'
NEW_MSG_ICON_WIN = 'icons/newmsg.ico'
BUSY_ICON_WIN = 'icons/busy.ico'
HELP_FILE = 'help' + os.sep + 'index.html'
VIEWED_MSGS_FILE = 'conf/data.pkl'

ABOUT_TEXT = "CERN Alerter \nVersion: 0.9b \n" + "Author: Svetoslav Zhelev \n" + u"\u00A9" + " 2010 CERN \n Support: helpdesk@cern.ch"

DEFAULT_CHECK_INTERVAL = int(5) * 60000 # 5 minutes

class CERNUIUtils:
    
    def isFeedURLValidWarningMsg(self, url, window):
        # test for valid feed
        try:
            reader = CERNNotifier.CERNFeedReader()
            reader.isFeedValid(url)
        except CERNNotifierFeedException, e:
            CERNConfiguration.getInstance().getLogger().exception(e.value)
            md = gtk.MessageDialog(window,
                              gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
                              gtk.BUTTONS_CLOSE, "Problems accessing feed. " + e.value)
            md.run()
            md.destroy()  

# Class responsible for showing the about dialog
class CERNNotifierAboutWindow:

    def __init__(self):
    
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title('CERN Notifier About')
        self.window.set_border_width(15)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_modal(True)
        self.window.set_resizable(False)
        self.window.set_property('skip-taskbar-hint', True)
        icon = gtk.gdk.pixbuf_new_from_file(ABOUT_ICON)
        self.window.set_icon_list((icon))
        
        table = gtk.Table(rows=2, columns=2, homogeneous=False)
        table.set_row_spacings(6)
        table.set_col_spacings(6)
        self.window.add(table)
        
        aboutPic = gtk.Image()
        pic = gtk.gdk.pixbuf_new_from_file(ABOUT_ICON)
        aboutPic.set_from_pixbuf(pic)
        table.attach(aboutPic, 0, 1, 0, 1, xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)
        
        aboutText = gtk.Label(ABOUT_TEXT)
        aboutText.set_line_wrap(True)
        table.attach(aboutText, 1, 2, 0, 1, xoptions=gtk.EXPAND, yoptions=gtk.EXPAND)
        
        #Add ok and quit buttons
        bbox = gtk.HButtonBox()
        bbox.set_layout(gtk.BUTTONBOX_END)
        bbox.set_spacing(8)
        
        ok_btn = gtk.Button(stock=gtk.STOCK_OK)
        ok_btn.connect("clicked", self.onOk)
        ok_btn.set_flags(gtk.CAN_DEFAULT)
        self.window.set_default(ok_btn)
        
        bbox.add(ok_btn)
        
        table.attach(bbox, 1, 2, 4, 5)
        
        self.window.set_default(ok_btn)
        table.show_all()
    
    def show(self):
        self.window.show()
        gtk.main()
    
    def onOk(self, widget, callback_data=None):
        self.window.hide()
        gtk.main_quit()

# Class responsible for showing the feed user/password dialog
class CERNNotifierFeedAuthWindow:

    def __init__(self, feed):
        
        self.feed = feed
        self.feedDisabled = False
        
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title('CERN Notifier Feed Authentication Info')
        self.window.set_border_width(15)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_modal(True)
        self.window.set_resizable(False)
        self.window.set_property('skip-taskbar-hint', True)
        icon = gtk.gdk.pixbuf_new_from_file(ABOUT_ICON)
        self.window.set_icon_list((icon))
        
        table = gtk.Table(rows=6, columns=1, homogeneous=False)
        table.set_row_spacings(6)
        table.set_col_spacings(6)
        self.window.add(table)

#        utils = CERNUtils()
#        url = utils.getHostname(feed.url.strip())

#TODO: introduce new lines in the URL on every 34-40 characters
        modifiedURL = ""
        for char in feed.url.strip():
            modifiedURL += char
            if len(modifiedURL) % 60 == 0 :
                modifiedURL += "\n"
                
        labelInfo = gtk.Label("Authentication required. Feed URL: \n  " + modifiedURL)
#        labelUrl = gtk.Label(feed.url)
#        labelInfo.set_alignment(1, 0.5)

        tableInfo = gtk.Table(rows=1, columns=1, homogeneous=False)
        tableInfo.attach(labelInfo, 0, 1, 0, 1)
#        tableInfo.attach(labelUrl, 0, 1, 1, 2)

        table.attach(tableInfo, 0, 1, 0, 1)
        
        tableData = gtk.Table(rows=2, columns=2, homogeneous=False)
        labelUser = gtk.Label('Username:')
#        labelUser.set_alignment(1, 0.5)
        tableData.attach(labelUser, 0, 1, 0, 1)
        
        self.textUser = gtk.Entry(max=0)
        self.textUser.set_text(feed.authFeedUser)
        tableData.attach(self.textUser, 1, 2, 0, 1)
        
        labelPass = gtk.Label('Password:')
#        labelPass.set_alignment(1, 0.5)
        tableData.attach(labelPass, 0, 1, 1, 2)
        
        self.textPass = gtk.Entry(max=0)
        self.textPass.set_text(feed.authFeedPass)
        self.textPass.set_visibility(False)
        tableData.attach(self.textPass, 1, 2, 1, 2)
        
        table.attach(tableData, 0, 1, 1, 2, gtk.FILL, gtk.FILL)
        
        tableRadio = gtk.Table(rows=2, columns=2, homogeneous=False)
        
        # Create disable feed checkbox 
        buttonChb = gtk.CheckButton("Disable feed")
        buttonChb.connect("toggled", self.callbackCheckBox, not self.feedDisabled)
        table.attach(buttonChb, 0, 1, 3, 4)

        #Add ok and cancel buttons
        bbox = gtk.HButtonBox()
        bbox.set_layout(gtk.BUTTONBOX_END)
        bbox.set_spacing(2)
        
        ok_btn = gtk.Button(stock=gtk.STOCK_OK)
        ok_btn.connect("clicked", self.onOk)
        ok_btn.set_flags(gtk.CAN_DEFAULT)
        self.window.set_default(ok_btn)
        
        close_btn = gtk.Button(stock=gtk.STOCK_CANCEL)
        close_btn.connect("clicked", self.onCancel)
        
        bbox.add(ok_btn)
        bbox.add(close_btn)
        
        table.attach(bbox, 0, 1, 5, 6)
        
        self.window.set_default(ok_btn)
        table.show_all()
    
    def show(self):
        self.window.show()
        #gtk.main()
    
    def getUser(self):
        return self.textUser.get_text().strip()
    
    def getPass(self):
        return self.textPass.get_text()
    
    def onOk(self, widget, callback_data=None):
        
        self.feed.authFeedUser = self.getUser()
        self.feed.authFeedPass = self.getPass()
        self.feed.authRequired = True
        
        self.feed.disabled = self.feedDisabled
        
        CERNConfiguration.getInstance().saveConfig()
        
        self.window.hide()
        #gtk.main_quit()
    
    def onCancel(self, widget, callback_data=None):
        self.window.hide()
        #gtk.main_quit()
        
    def callbackCheckBox(self, widget, data=None):
        if (data != None):
            self.feedDisabled = data

        
# Class responsible for showing the configuration window
class CERNNotifierConfigWindow:
    
    WINDOW_WIDTH = 600
    WINDOW_HEIGHT = 350
    
    def __init__(self):
        
        self.infoModel = None
        self.displayModel = None
        
        self.rssName = None
        self.rssURL = None
        self.username = None
        self.computername = None
        
        self.textRSSName = None
        self.text_rssURL = None
        self.textUser = None
        
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title('Preferences')
        self.window.set_border_width(5)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_modal(True)
        self.window.set_resizable(False)
        self.window.set_property('skip-taskbar-hint', True)
        icon = gtk.gdk.pixbuf_new_from_file(DEFAULT_ICON)
        self.window.set_icon_list((icon))
        
        table = gtk.Table(rows=2, columns=1, homogeneous=False)
        table.set_row_spacings(6)
        table.set_col_spacings(6)
        self.window.add(table)
        
        notebook = gtk.Notebook()
        notebook.set_tab_pos(gtk.POS_TOP)
        notebook.set_show_border(True)
        notebook.set_show_tabs(True)
        notebook.set_scrollable(False)
        
        table.attach(notebook, 0, 1, 0, 1)
        notebook.show()
        
        frame = gtk.Frame()
        frame.set_border_width(10)
        frame.set_size_request(CERNNotifierConfigWindow.WINDOW_WIDTH,
                               CERNNotifierConfigWindow.WINDOW_HEIGHT)
        frame.show()
        
        pageLabel = gtk.Label("Feeds Configuration")
        pageLabel.show()
        
        # create RSS grid
        configuration = CERNConfiguration.getInstance()
        
        frame.add(self.__createRssConfigPage(configuration))
        
        notebook.append_page(frame, pageLabel)
        
        frame2 = gtk.Frame()
        frame2.set_border_width(10)
        frame2.set_size_request(CERNNotifierConfigWindow.WINDOW_WIDTH,
                                CERNNotifierConfigWindow.WINDOW_HEIGHT)
        frame2.show()
        
        frame2.add(self.__createPreferencesConfigPage(configuration))
        
        pageLabel2 = gtk.Label("Base Configuration")
        pageLabel2.show()
        
        notebook.append_page(frame2, pageLabel2)
        
        notebook.set_current_page(0)
        
        #Add ok and quit buttons
        bbox = gtk.HButtonBox()
        bbox.set_layout(gtk.BUTTONBOX_END)
        bbox.set_spacing(8)
        
        saveBtn = gtk.Button(stock=gtk.STOCK_SAVE)
        saveBtn.connect("clicked", self.onSave)
        saveBtn.set_flags(gtk.CAN_DEFAULT)
        self.window.set_default(saveBtn)
        
        close_btn = gtk.Button(stock=gtk.STOCK_CANCEL)
        close_btn.connect("clicked", self.on_cancel)
        
        bbox.add(saveBtn)
        bbox.add(close_btn)
        
        table.attach(bbox, 0, 2, 1, 2)
        
        self.window.set_default(saveBtn)
        table.show_all()
    
    # Create RSS configuration page    
    def __createRssConfigPage(self, config):
        tableAddRSS = gtk.Table(rows=9, columns=5, homogeneous=True)
        tableAddRSS.set_row_spacings(6)
        tableAddRSS.set_col_spacings(6)
        
        self.labelRSSName = gtk.Label('RSS Name:')
        self.labelRSSName.set_alignment(1, 0.5)
        tableAddRSS.attach(self.labelRSSName, 0, 1, 0, 1)
        
        self.textRSSName = gtk.Entry(max=0)
        self.textRSSName.set_text("")
        tableAddRSS.attach(self.textRSSName, 1, 5, 0, 1)
        
        self.label_rssURL = gtk.Label('RSS URL:')
        self.label_rssURL.set_alignment(1, 0.5)
        tableAddRSS.attach(self.label_rssURL, 0, 1, 1, 2)
        
        self.text_rssURL = gtk.Entry(max=0)
        self.text_rssURL.set_text("")
        tableAddRSS.attach(self.text_rssURL, 1, 5, 1, 2)
        
        addBtn = gtk.Button(stock=gtk.STOCK_ADD)
        addBtn.connect("clicked", self.onAdd)
        addBtn.set_flags(gtk.CAN_DEFAULT)
        tableAddRSS.attach(addBtn, 4, 5, 2, 3)
        
        # create RSS tree grid
        tableAddRSS.attach(self.buildTreeView(config.feeds), 0, 5, 3, 8,
                           gtk.FILL | gtk.EXPAND, gtk.FILL | gtk.EXPAND, 1, 1)
        
        delBtn = gtk.Button(stock=gtk.STOCK_DELETE)
        delBtn.connect("clicked", self.onDel)
        delBtn.set_flags(gtk.CAN_DEFAULT)
        tableAddRSS.attach(delBtn, 4, 5, 8, 9)        
        
        tableAddRSS.show_all()
        
        return tableAddRSS
    
    # Create preferences configuration page    
    def __createPreferencesConfigPage(self, config):
        tablePreferences = gtk.Table(rows=9, columns=5, homogeneous=True)
        tablePreferences.set_row_spacings(6)
        tablePreferences.set_col_spacings(6)
        
        self.labelUser = gtk.Label('Username:')
        self.labelUser.set_alignment(1, 0.5)
        tablePreferences.attach(self.labelUser, 0, 1, 0, 1)
        
        self.textUser = gtk.Entry(max=0)
        self.textUser.set_text(config.username)
        tablePreferences.attach(self.textUser, 1, 5, 0, 1)
        
        self.labelPass = gtk.Label('Computername:')
        self.labelPass.set_alignment(1, 0.5)
        tablePreferences.attach(self.labelPass, 0, 1, 1, 2)
        
        self.textPass = gtk.Entry(max=0)
        self.textPass.set_text(config.computername)
        tablePreferences.attach(self.textPass, 1, 5, 1, 2)
        
        tablePreferences.show_all()
        
        return tablePreferences
        
    def buildTreeView(self, feeds):
        self.infoModel = InfoModel(feeds)
        self.displayModel = DisplayModel(self.window)
        view = self.displayModel.makeView(self.infoModel.get_model())
        return view
    
    def show(self):
        self.window.show()
        gtk.main()
    
    def getRSSName(self):
        return self.textRSSName.get_text().strip()
    
    def setRSSName(self, value):
        return self.textRSSName.set_text(value)
    
    def get_rssURL(self):
        return self.text_rssURL.get_text().strip()
    
    def set_rssURL(self, value):
        return self.text_rssURL.set_text(value)
    
    def getUsername(self):
        return self.textUser.get_text().strip()
    
    def getComputername(self):
        return self.textPass.get_text().strip()

    # add new RSS to configuration 
    def onAdd(self, widget, callback_data=None):
        if (self.getRSSName().strip() != '' and self.get_rssURL().strip() != ''):
            cernutils = CERNUIUtils()
            cernutils.isFeedURLValidWarningMsg(self.get_rssURL().strip(), self.window)
            self.infoModel.get_model().append(None, (self.getRSSName(), True, True, False, False, False,
                                                     True, self.get_rssURL(), False, False, False))
        self.set_rssURL("")
        self.setRSSName("")
    
    # delete RSS from configuration    
    def onDel(self, widget, callback_data=None):
        model, rows = self.displayModel.getGridView().get_selection().get_selected_rows()
        for row in rows:
            # CERN mandatory feeds cannot be deleted 
            if (self.infoModel.get_model()[row][8] == False): 
                self.infoModel.get_model().remove(model.get_iter(row))
            else:
                md = gtk.MessageDialog(self.window,
                          gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,
                          gtk.BUTTONS_CLOSE, "CERN Feed cannot be deleted")
                md.run()
                md.destroy()

    # save RSS feed configuration    
    def onSave(self, widget, callback_data=None):
        self.window.hide()
        feeds = self.__applyChangesToFeeds(self.infoModel.tree_store)
        
        # Change the current configuration settings
        configuration = CERNConfiguration.getInstance()
        # apply user and pass from old feeds
        for f in configuration.feeds:
            for current in feeds:
                if f.url == current.url:
                    current.authRequired = f.authRequired
                    current.authFeedUser = f.authFeedUser
                    current.authFeedPass = f.authFeedPass
                    break
             
        configuration.feeds = feeds
        configuration.username = self.getUsername()    
        configuration.computername = self.getComputername()
        configuration.saveConfig()
        
        gtk.main_quit()

    # Get a list with the current feeds
    def __applyChangesToFeeds(self, model):
        feeds = []
        for tmp in model:
            feed = CERNNotifier.CERNFeedConfiguration()
            feed.name = tmp[0] 
            feed.balloonForInfoMsg = tmp[1] 
            feed.balloonForImportantMsg = tmp[2] 
            feed.balloonForAlertMsg = tmp[3] 
            feed.popupForInfoMsg = tmp[4] 
            feed.popupForImportantMsg = tmp[5] 
            feed.popupForAlertMsg = tmp[6] 
            feed.url = tmp[7]
            feed.mandatory = tmp[8] 
            feed.CERNFeed = tmp[9]
            feed.disabled = tmp[10]
            feeds.append(feed)
        return feeds
    
    # Configuration window cancel button is pressed            
    def on_cancel(self, widget, callback_data=None):
        self.window.hide()
        gtk.main_quit()

# The model class holds the information we want to display
class InfoModel:
    
    def __init__(self, feeds):
        # Set up and populate our gtk.TreeStore
        self.tree_store = gtk.TreeStore(gobject.TYPE_STRING,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_STRING,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN,
                                        gobject.TYPE_BOOLEAN)
        # we form a simple tree.
        self.feeds = feeds
        for feed in feeds:
            self.tree_store.append(None, (feed.name.strip(), feed.balloonForInfoMsg, feed.balloonForImportantMsg, feed.balloonForAlertMsg,
                                          feed.popupForInfoMsg, feed.popupForImportantMsg, feed.popupForAlertMsg, feed.url.strip(),
                                          feed.mandatory, feed.CERNFeed, feed.disabled))
            
        self.printFeeds()
        return
    
    def printFeeds(self):
        CERNConfiguration.getInstance().getLogger().debug("Current feeds: ")
        for feed in self.feeds:
            CERNConfiguration.getInstance().getLogger().debug(feed)
    
    # Return the model
    def get_model(self):
        if self.tree_store:
            return self.tree_store
        else:
            return None

# Displays the Info_Model model in a gridView
class DisplayModel:
    
    def __init__(self, window):
        self.gridView = None
        self.window = window

    # Form a gridView for the Tree Model    
    def makeView(self, model):
        self.gridView = gtk.TreeView(model)
        # setup the text cell renderer and allows these cells to be edited.
        self.renderer = gtk.CellRendererText()
        self.renderer.set_property('editable', True)
        self.renderer.connect('edited', self.col0_edited_cb, model)
        
        self.rendererURL = gtk.CellRendererText()
        self.rendererURL.set_property('editable', True)
        self.rendererURL.connect('edited', self.col7_edited_cb_URL, model)

        # The toggle cellrenderer is setup and we allow it to be changed (toggled) by the user.
        self.renderer1 = gtk.CellRendererToggle()
        self.renderer1.set_property('activatable', True)
        self.renderer1.connect('toggled', self.col1_toggled_cb1, model)
        self.renderer2 = gtk.CellRendererToggle()
        self.renderer2.set_property('activatable', True)
        self.renderer2.connect('toggled', self.col1_toggled_cb2, model)
        self.renderer3 = gtk.CellRendererToggle()
        self.renderer3.set_property('activatable', True)
        self.renderer3.connect('toggled', self.col1_toggled_cb3, model)
        self.renderer4 = gtk.CellRendererToggle()
        self.renderer4.set_property('activatable', True)
        self.renderer4.connect('toggled', self.col1_toggled_cb4, model)
        self.renderer5 = gtk.CellRendererToggle()
        self.renderer5.set_property('activatable', True)
        self.renderer5.connect('toggled', self.col1_toggled_cb5, model)
        self.renderer6 = gtk.CellRendererToggle()
        self.renderer6.set_property('activatable', True)
        self.renderer6.connect('toggled', self.col1_toggled_cb6, model)
        
        self.renderer8 = gtk.CellRendererToggle()
        self.renderer8.set_property('activatable', True)
        self.renderer8.connect('toggled', self.col1_toggled_cb8, model)
        self.renderer9 = gtk.CellRendererToggle()
        self.renderer9.set_property('activatable', True)
        self.renderer9.connect('toggled', self.col1_toggled_cb9, model)        
        
        self.renderer10 = gtk.CellRendererToggle()
        self.renderer10.set_property('activatable', True)
        self.renderer10.connect('toggled', self.col1_toggled_cb10, model)    
        # Connect column0 of the display with column 0 in our list model             
        # The renderer will then display whatever is in column 0 of our model .
        self.column0 = gtk.TreeViewColumn("Name", self.renderer, text=0)
        self.column7 = gtk.TreeViewColumn("URL", self.rendererURL, text=7)   
        self.column7.set_visible(True)   
        # hide these columns because the functionality is not yet implemented
        self.column1 = gtk.TreeViewColumn("Balloon for info messages", self.renderer1)
        self.column1.add_attribute(self.renderer1, "active", 1)
        self.column1.set_visible(False)
        self.column2 = gtk.TreeViewColumn("Balloon for important messages", self.renderer2)
        self.column2.add_attribute(self.renderer2, "active", 2)
        self.column2.set_visible(False)
        self.column3 = gtk.TreeViewColumn("Balloon for alert messages", self.renderer3)
        self.column3.add_attribute(self.renderer3, "active", 3)
        self.column3.set_visible(False)
        self.column4 = gtk.TreeViewColumn("Popup for info messages", self.renderer4)
        self.column4.add_attribute(self.renderer4, "active", 4)
        self.column4.set_visible(False)
        self.column5 = gtk.TreeViewColumn("Popup for important messages", self.renderer5)
        self.column5.add_attribute(self.renderer5, "active", 5)
        self.column5.set_visible(False)
        self.column6 = gtk.TreeViewColumn("Popup for alert messages", self.renderer6)
        self.column6.add_attribute(self.renderer6, "active", 6)
        self.column6.set_visible(False)
        
        # Hide these by default. The user cannot change these private values
        self.column8 = gtk.TreeViewColumn("Mandatory", self.renderer8)
        self.column8.add_attribute(self.renderer8, "active", 8)
        self.column8.set_visible(False)
        self.column9 = gtk.TreeViewColumn("CERN Feed", self.renderer9)
        self.column9.add_attribute(self.renderer9, "active", 9)
        self.column9.set_visible(False)   
        
        self.column10 = gtk.TreeViewColumn("Disabled", self.renderer10)
        self.column10.add_attribute(self.renderer10, "active", 10)
        self.column10.set_visible(True)
        
        self.gridView.append_column(self.column10)        
        self.gridView.append_column(self.column0)
        self.gridView.append_column(self.column1)
        self.gridView.append_column(self.column2)
        self.gridView.append_column(self.column3)
        self.gridView.append_column(self.column4)
        self.gridView.append_column(self.column5)
        self.gridView.append_column(self.column6)
        self.gridView.append_column(self.column7)
        
        self.gridView.append_column(self.column8)
        self.gridView.append_column(self.column9)
        
        self.scrolledWindow = gtk.ScrolledWindow()
        self.scrolledWindow.add(self.gridView)
        
        return self.scrolledWindow
    
    def getGridView(self):
        return self.gridView
    
    def col0_edited_cb(self, cell, row, new_text, model):
        model[row][0] = new_text
        return

    def col7_edited_cb_URL(self, cell, row, new_text, model):
        model[row][7] = new_text
        cernutils = CERNUIUtils()
        cernutils.isFeedURLValidWarningMsg(new_text, self.window)  
        return
        
    def col1_toggled_cb1(self, cell, row, model):
        model[row][1] = not model[row][1]
        return
    
    def col1_toggled_cb2(self, cell, row, model):
        model[row][2] = not model[row][2]
        return
    
    def col1_toggled_cb3(self, cell, row, model):
        model[row][3] = not model[row][3]
        return
    
    def col1_toggled_cb4(self, cell, row, model):
        model[row][4] = not model[row][4]
        return
    
    def col1_toggled_cb5(self, cell, row, model):
        model[row][5] = not model[row][5]
        return
    
    def col1_toggled_cb6(self, cell, row, model):
        model[row][6] = not model[row][6]
        return

    def col1_toggled_cb8(self, cell, row, model):
        model[row][8] = not model[row][8]
        return
    
    def col1_toggled_cb9(self, cell, row, model):
        model[row][9] = not model[row][9]
        return

    def col1_toggled_cb10(self, cell, row, model):
        model[row][10] = not model[row][10]
        return    
        
# Class responsible for handling tray icon and menu    
class CERNNotifierTrayIcon:

    def __init__(self, interval):
        self._lock = threading.RLock()
        
        # init some class properties
        self.notifier = CERNNotifier.CERNFeedReader()
        self.interval = interval
        self.balloon = None
        self.checking = False
        self.currentMsg = None
        self.viewedMsg = dict()
        self.linkToFeedTitle = dict()
        self.newmsgs = []
        
        # load viewedmsgs from hdd
        config = CERNConfiguration.getInstance()
        pklFile = config.loadFromUsersHomeDir(VIEWED_MSGS_FILE)
        if(os.path.isfile(pklFile)):
            pkl_file = open(pklFile, 'r')
            self.viewedMsg = pickle.load(pkl_file)
            pkl_file.close()
        
        #load the three icons
        self.default_icon = gtk.gdk.pixbuf_new_from_file_at_size(os.path.abspath(DEFAULT_ICON), 24, 24)
        self.new_msg_icon = gtk.gdk.pixbuf_new_from_file_at_size(os.path.abspath(NEW_MSG_ICON), 24, 24)
        self.busy_icon = gtk.gdk.pixbuf_new_from_file_at_size(os.path.abspath(BUSY_ICON), 24, 24)

        # ideally the menu initialization for different platforms should be unified. However, I do not have time and desire to dig around
        if is_windows:
            self.wnd = gtk.Window()
            self.wnd.realize()
            self.win32ext = GTKWin32Ext(self.wnd)

            hinst = win32gui.GetModuleHandle(None)
            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
            hicon = win32gui.LoadImage(hinst, os.path.abspath(DEFAULT_ICON_WIN), win32con.IMAGE_ICON, 0, 0, icon_flags)

            self.win32ext.add_notify_icon(hicon)
            #self.win32ext.set_notify_icon(os.path.abspath(DEFAULT_ICON_WIN))			

            # GTK menus from the notify icon!
            menu = gtk.Menu()

            menu_item = gtk.ImageMenuItem(gtk.STOCK_HELP, 'Help')
            menu_item.connect('activate', self.onHelp)
            menu.append(menu_item)
                
            menu_item = gtk.ImageMenuItem(gtk.STOCK_ABOUT, 'About')
            menu_item.connect('activate', self.onAbout)
            menu.append(menu_item)
                
            menu_item = gtk.ImageMenuItem(gtk.STOCK_REFRESH, 'Refresh')
            menu_item.connect('activate', self.onCheckNow)
            menu.append(menu_item)

            menu_item = gtk.ImageMenuItem(gtk.STOCK_CDROM, 'History')
            menu_item.connect('activate', self.onHistory)
            menu.append(menu_item)

            menu_item = gtk.ImageMenuItem(gtk.STOCK_PREFERENCES, 'Preferences')
            menu_item.connect('activate', self.onPreferences)
            menu.append(menu_item)

            menu_item = gtk.ImageMenuItem(gtk.STOCK_QUIT, 'Quit')
            menu_item.connect('activate', self.onQuit)
            menu.append(menu_item)

            menu.show_all()
            self.win32ext.notify_icon.menu = menu

            # Set up the callback messages
            self.win32ext.message_map({
                WM_TRAYMESSAGE: self.on_notifyicon_activity
                })            
        else:
            #create the tray icon
            self.tray_icon = gtk.StatusIcon()
            self.tray_icon.connect('activate', self.onCheckNow)
            self.tray_icon.connect('popup-menu', self.onTrayIconClick)		

            self.tray_icon.set_from_pixbuf(self.default_icon)

            # Create the main menu actions
            self.actiongroup = gtk.ActionGroup("Main")
            self.actiongroup.add_actions([
			('Help', gtk.STOCK_HELP, None, None, None, self.onHelp),
			('About', gtk.STOCK_ABOUT, None, None, None, self.onAbout),
			('Refresh', gtk.STOCK_REFRESH, None, None, None, self.onCheckNow),
			('History', gtk.STOCK_CDROM, "History", "h", "Show history of your CERN messages.", self.onHistory),
			('Preferences', gtk.STOCK_PREFERENCES, None, None, None, self.onPreferences),
			('Quit', gtk.STOCK_QUIT, None, None, None, self.onQuit)
			])

            self.uimanager = gtk.UIManager()
            self.uimanager.insert_action_group(self.actiongroup, 0)
            self.uimanager.add_ui_from_string(\
			"""
			<ui>
			<popup name='TrayMenu'>
			<menuitem action='Help' />
			<menuitem action='About' />
			<menuitem action='Refresh' />
			<menuitem action='History' />
			<menuitem action='Preferences' />
			<menuitem action='Quit' />
			</popup>
			</ui>
			""")

        # prevents UI from freezing
        #while gtk.events_pending():
            #gtk.main_iteration(True)
    
        # check for new feeds every "interval"
        self.timer = gobject.timeout_add(self.interval, self.onCheckNow) 

    def on_notifyicon_activity(self, hwnd, message, wparam, lparam):
        if lparam == WM_RBUTTONUP:
            self.win32ext.notify_icon.menu.popup(None, None, None, 0, 0)
        elif lparam == WM_LBUTTONUP:
            #self.win32ext.notify_icon.menu.popdown()      
            self.onCheckNow()

    # Show popup menu
    def onTrayIconClick(self, status_icon, button, activate_time):
        self.uimanager.get_widget("/TrayMenu").popup(None, None, None, button, activate_time)
    
    # Show help window
    def onHelp(self, event=None):
        webbrowser.open_new(HELP_FILE)

    # Show about dialog
    def onAbout(self, event=None):
        self.aboutWindow = CERNNotifierAboutWindow()
        self.aboutWindow.show()

    # Show preferences window
    def onPreferences(self, event=None):
        cfg_dlg = CERNNotifierConfigWindow()
        cfg_dlg.show()
    
    # Show CERN feed history 
    # NOTE: Main CERN feed is determined by feed.mandatory . 
    # There should be only 1 mandatory feed. This code will show 
    # the history for the first found mandatory feed  
    def onHistory(self, event=None):
        url = ""
        configuration = CERNConfiguration.getInstance()
        for feed in configuration.feeds:
            if (feed.mandatory):
                utils = CERNUtils()
                url = utils.buildCERNFeedURL(feed.url.strip())
                break
        webbrowser.open_new(url)
        
    # Exit the application    
    def onQuit(self, event=None):
        gtk.main_quit()
        sys.exit(0)
    
    # Check for new RSS feeds
    def onCheckNow(self, checkForNewFeeds=True, event=None):
#        self._lock.acquire()
        # 2 try blocks to ensure python 2.4 compatibility
        try:
            try:
                # simple lock
                if self.checking:
                    return
                else:
                    self.checking = True
                    # change the taskbar icon
                    #if (is_windows):
                    #	self.win32ext.set_notify_icon(os.path.abspath(BUSY_ICON_WIN))
                    #else:
                    if (not is_windows):
                        self.tray_icon.set_from_pixbuf(self.busy_icon)
            
                # prevents UI from freezing
                while gtk.events_pending():
                    gtk.main_iteration(True)
                
                if(checkForNewFeeds):
                    self.newmsgs = []
                    self.currentMsg = None
                    self.linkToFeedTitle = dict()
                    configuration = CERNConfiguration.getInstance()
                    
                    for feed in configuration.feeds:
                        # get the new feeds
                        try:
                            if (feed.disabled):
                                CERNConfiguration.getInstance().getLogger().info("Feed with URL: " + feed.url.strip() + " is disabled. Skipping.")
                                continue
                            CERNConfiguration.getInstance().getLogger().info("Get feeds from URL: " + feed.url.strip())
                            tmpMsgs = self.notifier.getNewFeeds(feed)
                            for msg in tmpMsgs:
                                if(msg.link not in self.viewedMsg):
                                    expDate = None
                                    # if it is a CERN feed then it should have expirationDate
                                    if (msg.expirationDate != 'None'):
                                        expDate = int(mktime(strptime(msg.expirationDate, '%a, %d %b %Y %H:%M:%S %Z')))
                                    # if it is CERN feed and expiration date not passed then do not add it to new messages array
                                    if (expDate != None and (expDate - int(time())) < 0):
                                        CERNConfiguration.getInstance().getLogger().debug("Expiration date passed for feed " + msg.link)
                                        continue
                                    self.newmsgs.append(msg)
                                    self.linkToFeedTitle[msg.link] = feed.name
                                else:
                                    CERNConfiguration.getInstance().getLogger().debug(msg.link + " already viewed.")  
                                     
                        except CERNNotifierFeedAuthException, e:
                            CERNConfiguration.getInstance().getLogger().info("Feed requires authentication. Please provide valid credentials. Feed: " + feed.url.strip())
                            authWindow = CERNNotifierFeedAuthWindow(feed)
                            authWindow.show()
            
                        except CERNNotifierFeedException, e:
                            # Show warnings only on linux/macos. There is no image in win balloons
                            if (not is_windows):
                                CERNConfiguration.getInstance().getLogger().exception("Cannot read feed. " + e.value)
                                imageURI = 'file://' + os.path.abspath(os.path.curdir) + '/icons/warning_balloon.gif'
                                self.balloon = pynotify.Notification("Cannot read feed.", e.value, imageURI)
                                # balloon timeout 
                                self.balloon.set_timeout(10 * 1000) # 10 seconds
                                self.balloon.attach_to_status_icon(self.tray_icon)

                                self.balloon.set_urgency(pynotify.URGENCY_NORMAL)

                                # show the balloon
                                self.balloon.show()

                if len(self.newmsgs) > 0:
                    self.currentMsg = self.newmsgs.pop(0)
                    
                    title = 'News from the Darkside there is!'
                    if(self.currentMsg.link in self.linkToFeedTitle):
                        title = self.linkToFeedTitle[self.currentMsg.link].strip()
                    
                    #body = '<b>%s</b>\n%s' % (self.currentMsg.title, self.currentMsg.body)
                    #body = '<b>%s</b>' % (self.currentMsg.title)
                    body = self.currentMsg.title
                    
                    CERNConfiguration.getInstance().getLogger().debug("Showing feed " + title + ", " + body)
                    
                    imageURI = ""
                    if (self.currentMsg.link.lower().find("cern.ch") != -1):
                        imageURI = 'file://' + os.path.abspath(os.path.curdir) + '/icons/cern_balloon.jpg'
                    if (is_windows):
                        #hinst = win32gui.GetModuleHandle(None)
                        #icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
                        #hicon = win32gui.LoadImage(hinst, os.path.abspath('icons/cern_balloon.ico'), win32con.IMAGE_ICON, 0, 0, icon_flags)					
                        self.win32ext.show_balloon_tooltip(title, body, 30)
                        self.__msgViewed()
                    else:
                        # icon
                        self.balloon = pynotify.Notification(title, body, imageURI)
                        # balloon timeout 
                        self.balloon.set_timeout(30 * 1000) # 30 seconds
                        self.balloon.attach_to_status_icon(self.tray_icon)
                        #            icon = self.__getIconForMsg()
                        #            self.balloon.set_icon_from_pixbuf(icon)
                        self.balloon.set_urgency(pynotify.URGENCY_NORMAL)
                        self.balloon.add_action("view", "Yes, View", self.view_cb)
                        self.balloon.add_action("ignore", "Ignore", self.ignore_cb)
                        self.balloon.add_action("ignoreAll", "Ignore All", self.ignoreAll_cb)			
                        # show the balloon
                        self.balloon.show()

                    self.__fixToolbarIconAndMsg()
            except Exception, e:
                CERNConfiguration.getInstance().getLogger().exception("Exception while reading/showing feed.")
        finally:
            self.__fixToolbarIconAndMsg()
            self.checking = False
#            self._lock.release()
            
        # Keep timeout alive
        return True

    # update toolbar icon and tooltip
    def __fixToolbarIconAndMsg(self):
        # we may have 0 msgs in newmsgs but the user might have not viewed the last one
        if len(self.newmsgs) > 0 or (self.currentMsg != None and not self.viewedMsg.has_key(self.currentMsg.link)):
            #if (is_windows):
                #self.win32ext.set_notify_icon(os.path.abspath(NEW_MSG_ICON_WIN))
            #else:
            if (not is_windows):
                self.tray_icon.set_from_pixbuf(self.new_msg_icon)
            # messages are popped from newmsgs array so actually the unread msgs are self.newmsgs + 1 
            if len(self.newmsgs) == 0 and self.currentMsg != None and not self.viewedMsg.has_key(self.currentMsg.link):
                if (is_windows):
                    self.win32ext.set_tooltip('1 new message!')
                else:
                    self.tray_icon.set_tooltip('1 new message!')

            else:
                length = len(self.newmsgs) + 1
                if (is_windows):
                    self.win32ext.set_tooltip('%d new messages!' % length)
                else:
                    self.tray_icon.set_tooltip('%d new messages!' % length)
        else:
            if (is_windows):
                #self.win32ext.set_notify_icon(os.path.abspath(DEFAULT_ICON_WIN))
                self.win32ext.set_tooltip('No new messages.')
            else:		
                self.tray_icon.set_from_pixbuf(self.default_icon)
                self.tray_icon.set_tooltip('No new messages.')
            
    # get appropriate icon for different type messages (Error, Important, Information)   
    def __getIconForMsg(self):
        helper = gtk.Button()
        #if (self.currentMsg.)
        return helper.render_icon(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)

    # view balloon button
    def view_cb(self, b, action):
        assert action == "view"
        webbrowser.open_new(self.currentMsg.link)
        self.__msgViewed()
    
    # ignore balloon button
    def ignore_cb(self, b, action):
        assert action == "ignore"
        self.closeBalloon_cb(self.balloon)
        
    # ignore All balloon button
    def ignoreAll_cb(self, b, action):
        assert action == "ignoreAll"
        for msg in self.newmsgs:
            self.viewedMsg[msg.link] = True
        self.viewedMsg[self.currentMsg.link] = True
        self.newmsgs = []
        self.__fixToolbarIconAndMsg()
        self.__saveMsgs()
        b.close()     
    
    def closeBalloon_cb(self, b):
        self.__msgViewed()
        b.close()
        
    def __msgViewed(self):
        self.viewedMsg[self.currentMsg.link] = True
        self.onCheckNow(False)
        self.__saveMsgs()
        
    # save newmsgs to hdd    
    def __saveMsgs(self):
        config = CERNConfiguration.getInstance()
        pklFile = config.loadFromUsersHomeDir(VIEWED_MSGS_FILE)
        output = open(pklFile, 'w')
        pickle.dump(self.viewedMsg, output)
        output.close()

    
if __name__ == '__main__':
    CERNConfiguration.getInstance().getLogger().info("########################")
    CERNConfiguration.getInstance().getLogger().info("# CERNAlerter started! #")
    CERNConfiguration.getInstance().getLogger().info("########################")
    
    tray_icon = CERNNotifierTrayIcon(DEFAULT_CHECK_INTERVAL)
    tray_icon.onCheckNow(True)
   
#    cfg_dlg = CERNNotifierConfigWindow()
#    cfg_dlg.show()

#    about = CERNNotifierAboutWindow()
#    about.show()
   
    gtk.main()
